/*---------------------------------------------------------------------------\
| Project:	Generic LV2 PIC18F458 CAN node firmware
|
| File:		can.h
|
| Descr.:	Includes for all CAN routines
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Created.
\---------------------------------------------------------------------------*/

#ifndef _CAN_H          // Don't include more than once (last line = #endif)
#define _CAN_H



//------------------------------------------------------------------------
// Typedefs and Enumerations
//------------------------------------------------------------------------


// Main structure for holding CAN Message ID's. Note that the 11 bit
// ID is stored in bits 15:5.

typedef union
{
    UINT16	id;

    struct		//  ASSUMING LITTLE ENDIAN!!!!!
    {
        UINT8 lowbyte;			// Bits 0 -  8 of the 16bit header
        UINT8 highbyte;			// Bits 8 - 15 of the 16bit header
    } bytes;

} CanId;



/*********************************************************************
 * CanHeader - PSAS PIC CAN Header Union
 *
 * This union provides abstract data type for CAN message id.
 * It is used for both 11-bit and 29-bit message identifiers.
 * There are multiple union members to be able to access individual
 * parts of it.
 *
 * Example:
 *
 * 	CanHeader foo;
 *
 * 	foo.Header = 		all 16 bits
 * 	foo.LowByte =		low 8 bits which is DLC, RTR and low ID
 * 	foo.HighByte = 		high 8 bits
 * 	foo.MsgId =			11 bits of ID
 *
 * FC declares a CAN header as:
 *
 *	typedef struct header_s
 *	{
 *		UINT16	msgid:11;
 *		UINT16	request:1;
 *		UINT16	length:4;
 *	} header_t;
 *
 * Note that if Length and request = 0, then the CanHeader can be
 * used as the 11bit CAN Message ID packed into the upper bits
 * of a 16bit value.
 ********************************************************************/


typedef union
{
    UINT16	Header;

    struct		//  ASSUMING LITTLE ENDIAN!!!!!
    {
        UINT16	Length:4;		// Bits 0 -  3: Number of data bytes in message
        UINT16	Request:1;		// Bits 4 -  4: Remote Frame Request bit
        UINT16	MsgIdL:3;		// Bits 5 - 7: CAN ID, Bit 5 = LSB, Bit15 = MSB --- IS THIS TRUE??!?!?! COMPILER DEPENDENT!!!
        UINT16	MsgIdH:8;		// Bits 8 - 15: CAN ID, Bit 5 = LSB, Bit15 = MSB --- IS THIS TRUE??!?!?! COMPILER DEPENDENT!!!
    } HeaderBits;

    struct
    {
        UINT8 LowByte;			// Bits 0 -  8 of the 16bit header
        UINT8 HighByte;			// Bits 8 - 15 of the 16bit header
    } HeaderBytes;

} CanHeader;



enum CAN_MODES
{
    CAN_MODE_BITS    = 0b11100000,  // Mask to access mode bits
    CAN_NORMAL_MODE  = 0b00000000,
    CAN_SLEEP_MODE   = 0b00100000,
    CAN_LOOP_MODE    = 0b01000000,
    CAN_LISTEN_MODE  = 0b01100000,
    CAN_CONFIG_MODE  = 0b10000000
};



enum CAN_BITRATE_TYPE
{
	CAN_BITRATE_125KBPS,
	CAN_BITRATE_1MBPS
};



/*********************************************************************
 *
 * enum CAN_TX_MSG_FLAGS
 *
 * This enumeration values define flags related to transmission of a
 * CAN message.  There could be more than one this flag
 * ANDed together to form multiple flags.
 *
 *********************************************************************/

enum CAN_TX_MSG_FLAGS
{
    CAN_TX_PRIORITY_BITS= 0b00000011,
    CAN_TX_PRIORITY_0   = 0b11111100,   // XXXXXX00
    CAN_TX_PRIORITY_1   = 0b11111101,   // XXXXXX01
    CAN_TX_PRIORITY_2   = 0b11111110,   // XXXXXX10
    CAN_TX_PRIORITY_3   = 0b11111111,   // XXXXXX11

    CAN_TX_FRAME_BIT    = 0b00001000,
    CAN_TX_STD_FRAME    = 0b11111111,   // XXXXX1XX
    CAN_TX_XTD_FRAME    = 0b11110111,   // XXXXX0XX

    CAN_TX_RTR_BIT      = 0b01000000,
    CAN_TX_NO_RTR_FRAME = 0b11111111,   // X1XXXXXX
    CAN_TX_RTR_FRAME    = 0b10111111    // X0XXXXXX
};


/*********************************************************************
 *
 * enum CAN_RX_MSG_FLAGS
 *
 * This enumeration values define flags related to reception of a CAN
 * message.  There could be more than one this flag
 * ANDed together to form multiple flags.
 * If a particular bit is set, corresponding meaning is TRUE or else
 * it will be FALSE.
 *
 * e.g.
 *      if (MsgFlag & CAN_RX_OVERFLOW)
 *      {
 *          // Receiver overflow has occured.  We have lost previous
 *          // message.
 *          ...
 *      }
 *
 ********************************************************************/

enum CAN_RX_MSG_FLAGS
{
    CAN_RX_FILTER_BITS  = 0b00000111,   // Use this to access filter
                                        // bits
    CAN_RX_FILTER_1     = 0b00000000,
    CAN_RX_FILTER_2     = 0b00000001,
    CAN_RX_FILTER_3     = 0b00000010,
    CAN_RX_FILTER_4     = 0b00000011,
    CAN_RX_FILTER_5     = 0b00000100,
    CAN_RX_FILTER_6     = 0b00000101,

    CAN_RX_OVERFLOW     = 0b00001000,   // Set if Overflowed else
                                        // cleared

    CAN_RX_INVALID_MSG  = 0b00010000,   // Set if invalid else
                                        // cleared

    CAN_RX_XTD_FRAME    = 0b00100000,   // Set if XTD message else
                                        // cleared

    CAN_RX_RTR_FRAME    = 0b01000000,   // Set if RTR message else
                                        // cleared

    CAN_RX_DBL_BUFFERED = 0b10000000    // Set if this message was
                                        // hardware double-buffered
};


/*********************************************************************
 *
 * enum CAN_MASK
 *
 * This enumeration values define mask codes.  Routine CANSetMask()
 * requires this code as one of its arguments.  These enumerations
 * must be used by itself i.e. it cannot be ANDed to form multiple
 * values.
 *
 * ADG 5/28/02 - Added RX_, set to B0 and B1
 ********************************************************************/

enum CAN_MASK
{
     CAN_RX_MASK_B0,
     CAN_RX_MASK_B1
};


/*********************************************************************
 *
 * enum CAN_FILTER
 *
 * This enumeration values define filter codes.  Routine CANSetFilter
 * requires this code as one of its arguments.  These enumerations
 * must be used by itself
 * i.e. it cannot be ANDed to form multiple values.
 *
 * ADG 5/28/02 - Added RX_, set to B0 and B1, F0 - 5 to be explicit
 ********************************************************************/

enum CAN_FILTER
{
    CAN_RX_FILTER_B0_F0,
    CAN_RX_FILTER_B0_F1,
    CAN_RX_FILTER_B1_F2,
    CAN_RX_FILTER_B1_F3,
    CAN_RX_FILTER_B1_F4,
    CAN_RX_FILTER_B1_F5
};



//------------------------------------------------------------------------
// Function Prototypes
//------------------------------------------------------------------------


// Initialize the CAN peripheral with the requested bitrate
// and setup the masks and filters on the receive buffers.

 void InitializeCan(enum CAN_BITRATE_TYPE canBitRate,
                    CanId canMaskB0,
                    CanId canMaskB1,
                    CanId CanFilterB0F0,
                    CanId CanFilterB0F1,
                    CanId CanFilterB1F2,
                    CanId CanFilterB1F3,
                    CanId CanFilterB1F4,
                    CanId CanFilterB1F5 );

// Do the nitty gritty time details to set up bit rates and bit timing

void SetCanTiming (enum CAN_BITRATE_TYPE canBitRate);


// Change to the specified CAN peripheral mode.
// WARNING: BLOCKING CALL. EVENTUALLY WILL EXIT AFTER TIMEOUT,
// BUT THAT'S NOT YET IMPLEMENTED.

void RequestCanMode(enum CAN_MODES mode);



// Set the CAN mask for RB0 or RB1 with the specified mask value

void CanSetMask (enum CAN_MASK canMaskNum, CanId mask);



// Set the CAN filter for F0 - F5 (RB0 or RB1) with the specified filter value

void CanSetFilter (enum CAN_FILTER canFilterNum, CanId filter);



/*********************************************************************
 * Function:        BOOL CANSendMessage(unsigned long id,
 *                                  UINT8 *Data,
 *                                  UINT8 DataLen,
 *                                  enum CAN_TX_MSG_FLAGS MsgFlags)
 *
 * PreCondition:    None
 *
 * Input:           id          - CAN message identifier.
 *                                Only 11 or 29 bits may be used
 *                                depending on standard or extended
 *                                message type.
 *                  Data        - Data bytes of upto 8 bytes in length
 *                  DataLen     - Data length from 1 thru 8.
 *                  MsgFlags    - One or CAN_TX_MSG_FLAGS values ANDed
 *                                together
 *
 * Output:          If at least one empty transmit buffer is found,
 *                  given message is queued to be transmitted. If none
 *                  found FALSE value is returned.
 *
 * Side Effects:    None
 *
 ********************************************************************/

//BOOL CANSendMessage(unsigned long id,
//                     UINT8 *Data,
//                     UINT8 DataLen,
//                     enum CAN_TX_MSG_FLAGS MsgFlags);

/*********************************************************************
 * Function:        BOOL CANReceiveMessage(unsigned long *id,
 *                                  UINT8 *Data,
 *                                  UINT8 *DataLen,
 *                                  enum CAN_RX_MSG_FLAGS *MsgFlags)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          id          - CAN message identifier.
 *                  Data        - Data bytes of upto 8 bytes in length
 *                  DataLen     - Data length from 1 thru 8.
 *                  MsgFlags    - One or CAN_RX_MSG_FLAGS values ANDed
 *                                together
 *
 * Output:          If at least one full receive buffer is found,
 *                  it is extrated and returned. If none found FALSE
 *                  value is returned.
 *
 * Side Effects:    None
 *
 ********************************************************************/

//BOOL CANReceiveMessage(unsigned long* id,
//                        UINT8 *Data,
//                        UINT8 *DataLen,
//                        enum CAN_RX_MSG_FLAGS *MsgFlags);



//void	CanTxIsr		(void);

//void	CanRxIsr		(void);

//void	CanErrorIsr		(void);






//------------------------------------------------------------------------
// #define'd Function Prototypes
//------------------------------------------------------------------------



// Set the operational mode, but do not wait for it to be implemented.
#define RequestCanModeNoWait(mode) (CANCON = mode)


// Get the operational mode.
#define CanGetMode() (CANCON & CAN_OP_MODE_BITS)


// Get the count of transmit errors (UINT8)
#define CANGetTxErrorCount()    (TXERRCNT)


// Get the count of receive errors (UINT8)
#define CanRxErrorCount()    (RXERRCNT)


// Check if the CAN peripheral mode is "Bus Off" due to errors
#define CanIsBusOff()	(COMSTATbits.TXB0)


// Check if the CAN transmit state is Error Passive
#define CanTxErrorPassive()	(COMSTATbits.TXBP)


// Check if the CAN receive state is Error Passive
#define CanRxErrorPassive()	(COMSTATbits.RXBP)


// Abort all CAN messages being transmitted
#define CanAbortAllTx()		(CANCONbits.ABAT = 1)


// Check if there's a received messsage
#define CanRxReady()		(RXB0CONbits.RXFUL || RXB1CONbits.RXFUL)


// Check if there's an empty transmit buffer
#define CanTxReady()		(!TXB0CONbits.TXREQ || \
							 !TXB1CONbits.TXREQ || \
							 !TXB2CONbits.TXREQ )

//------------------------------------------------------------------------


#endif      // _CAN_H

