/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				interrupts.c
|
| Description:		Interrupt Handlers.
|
| Global Routines:	InitializeInterrupts
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <pic18.h>    	// Include for PICC-18
#include "pic18fxx8.h"	// Register definitions for the PIC18F458
#include "project.h"	// Includes for the project (Globals + misc)
#include "main.h"		// Includes for the mainline
#include "ints.h"		// Includes for interrupts
#include "excepts.h"	// Includes for exception handling
#include "ports.h"		// Includes for the port peripherals
#include "timer2.h"		// Includes for the Timer2 peripheral
#include "serial.h"		// Includes for the serial peripheral (UART )
#include "can.h"		// Includes for the CAN peripheral


//------------------------------------------------------------------------
//  Local function prototypes
//------------------------------------------------------------------------


/*---------------------------------------------------------------------------\
| Function:		HighPriorityISR
|
| Purpose:		Handle all of the high priority interrupts
|
| Discussion:	Checks to see what interrupted us. Why all this weird #ifdef
|				stuff? Because the PIC can't due recursive calls, and we have
|				an error function ("ThrowWarning") with a mainline, high
|				priority, and low priority flavors. In order for the compiler
|				to not yell at us, we need to figure out which interrupts
|				are which priority and #ifdef the correct error functions.
|				Ugly, but it's the best we can do (for now).
|
|				Why check to see if the interrupt is enabled? Because
|				any interrupt gets us here, we need to check to see
|				which interrupt needs servicing. But how can we tell if
|				if the interrupt flag is set but the interrupt is disabled?
|				We have to manually check. Stupid, but true.
|
| Expects:		all interrupts to be disabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE HIGH PRIOIRTY INTERRUPT SERVICE ROUTINE
// USING THE "interrupt" KEYWORD.


void interrupt HighPriorityISR (void)
{
	static bit hpTest;

	hpTest = FALSE;

	#if	CANTXB2INTPRIORITY == HIGHPRIORITYINTERRUPT
	if ( CanTxb2InterruptPending == TRUE
		 && CanTxb2InterruptEnabled == TRUE )
	{
		CanTx2Isr();
		hpTest = TRUE;
	}
	#endif

	#if	CANTXB1INTPRIORITY == HIGHPRIORITYINTERRUPT
	if ( CanTxb1InterruptPending == TRUE
		 && CanTxb1InterruptEnabled == TRUE )
	{
		CanTx1Isr();
		hpTest = TRUE;
	}
	#endif

	#if	CANTXB0INTPRIORITY == HIGHPRIORITYINTERRUPT
	if ( CanTxb0InterruptPending == TRUE
		 && CanTxb0InterruptEnabled == TRUE )
	{
		CanTx0Isr();
		hpTest = TRUE;
	}
	#endif

	#if	TIMER2INTPRIORITY == HIGHPRIORITYINTERRUPT
	if ( Timer2InterruptPending == TRUE
		 && Timer2InterruptEnabled == TRUE )

	{
		Timer2Isr();
		hpTest = TRUE;
	}
	#endif

	#if	SERIALTXINTPRIORITY == HIGHPRIORITYINTERRUPT
	if ( SerialTxInterruptPending == TRUE
		 && SerialTxInterruptEnabled == TRUE )
	{
		SerialTxIsr();
		hpTest = TRUE;
	}
	#endif

	#if	SERIALRXINTPRIORITY == HIGHPRIORITYINTERRUPT
	if ( SerialRxInterruptPending == TRUE
		 && SerialRxInterruptEnabled == TRUE )

	{
		SerialRxIsr();
		hpTest = TRUE;
	}
	#endif

	// Throw a warning if we don't know why were interrupted.

	if (hpTest == FALSE)
		ThrowWarning_HP(WARN_ISR_UNKNOWN_HPI);
}



/*---------------------------------------------------------------------------\
| Function:		LowPriorityISR
|
| Purpose:		Handle all of the low prioirty interrupts
|
| Discussion:	Checks to see what interrupted us.
|
| Expects:		Low prioirty interrupts disabled but high prioirty may still
|				be enabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE LOW PRIOIRTY INTERRUPT SERVICE ROUTINE
// USING THE "interrupt low-priority" KEYWORDS.

void interrupt low_priority LowPriorityISR (void)
{
	static bit lpTest;

	lpTest = FALSE;

	#if	CANTXB2INTPRIORITY == LOWPRIORITYINTERRUPT
	if ( CanTxb2InterruptPending == TRUE
		 && CanTxb2InterruptEnabled == TRUE )
	{
		CanTx2Isr();
		lpTest = TRUE;
	}
	#endif

	#if	CANTXB1INTPRIORITY == LOWPRIORITYINTERRUPT
	if ( CanTxb1InterruptPending == TRUE
		 && CanTxb1InterruptEnabled == TRUE )
	{
		CanTx1Isr();
		lpTest = TRUE;
	}
	#endif

	#if	CANTXB0INTPRIORITY == LOWPRIORITYINTERRUPT
	if ( CanTxb0InterruptPending == TRUE
		 && CanTxb0InterruptEnabled == TRUE )
	{
		CanTx0Isr();
		lpTest = TRUE;
	}
	#endif

	#if	TIMER2INTPRIORITY == LOWPRIORITYINTERRUPT
	if ( Timer2InterruptPending == TRUE
		 && Timer2InterruptEnabled == TRUE )
	{
		Timer2Isr();
		lpTest = TRUE;
	}
	#endif

	#if	SERIALTXINTPRIORITY == LOWPRIORITYINTERRUPT
	if ( SerialTxInterruptPending == TRUE
		 && SerialTxInterruptEnabled == TRUE )
	{
		SerialTxIsr();
		lpTest = TRUE;
	}
	#endif

	#if	SERIALRXINTPRIORITY == LOWPRIORITYINTERRUPT
	if ( SerialRxInterruptPending == TRUE
		 && SerialRxInterruptEnabled == TRUE )
	{
		SerialRxIsr();
		lpTest = TRUE;
	}
	#endif

	// Throw a warning if we don't know why were interrupted.

	if (lpTest == FALSE)
		ThrowWarning_LP(WARN_ISR_UNKNOWN_LPI);
}





/*---------------------------------------------------------------------------\
| Function:		InitializeInterrupts
|
| Purpose:		Initialize the PIC's interrupts
|
| Discussion:	Huh?
|
| Expects:		All interupts to be off (after all, we were just reseted).
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializeInterrupts (void)
{
	// Turn on "high and low" priorities, not old peripheral/global

	EnablePrioritizedInts();

	// NOTE: All interrupts can be either low or high prioirity. Which one
	// they are is defined in "ints.h" by #defines. We still need to set the
	// correct bits, which we do here. Why the funny #ifdef? The lack of
	// recursion in PICs makes this best to do in pre-processor land. See
	// "ints.h" for more info.

	// Although the prioirty can be 0 (off), 1 (low) or 2 (high) we can only
	// set the prioirty bits for the PIC to 1 or 0; thus off and low are set
	// to 1 and high is set to 2.

	#if CANTXB2INTPRIORITY == HIGHPRIORITYINTERRUPT
	CanTxb2PrioritySetting = HIGHPRIORITY;
	#else
	CanTxb2PrioritySetting = LOWPRIORITY;
	#endif

	#if CANTXB1INTPRIORITY == HIGHPRIORITYINTERRUPT
	CanTxb1PrioritySetting = HIGHPRIORITY;
	#else
	CanTxb1PrioritySetting = LOWPRIORITY;
	#endif

	#if CANTXB0INTPRIORITY == HIGHPRIORITYINTERRUPT
	CanTxb0PrioritySetting = HIGHPRIORITY;
	#else
	CanTxb0PrioritySetting = LOWPRIORITY;
	#endif

	#ifdef TIMER2INTPRIORITY == HIGHPRIORITYINTERRUPT
	Timer2PrioritySetting = HIGHPRIORITY;
	#else
	Timer2PrioritySetting = LOWPRIORITY;
	#endif

	#ifdef SERIALTXPRIORITY == HIGHPRIORITYINTERRUPT
	SerialTxPrioritySetting = HIGHPRIORITY;
	#else
	SerialTxPrioritySetting = LOWPRIORITY;
	#endif

	#ifdef SERIALRXPRIORITY == HIGHPRIORITYINTERRUPT
	SerialRxPrioritySetting = HIGHPRIORITY;
	#else
	SerialRxPrioritySetting = LOWPRIORITY;
	#endif


	// Turn on ints and away we go!

	EnableHighPriorityInts();
	EnableLowPriorityInts();
}

